import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useEffect, useMemo, useState } from 'react';
import { classNames } from '../utils/classnames.js';
const NODE_PADDING_LEFT = 12;
const DEFAULT_HIDDEN_FILES = [/\/node_modules\//];
export function FileTree({ files, onFileSelect, selectedFile, hideRoot, scope, hiddenFiles, className }) {
    const computedHiddenFiles = useMemo(() => [...DEFAULT_HIDDEN_FILES, ...(hiddenFiles ?? [])], [hiddenFiles]);
    const fileList = useMemo(() => buildFileList(files, hideRoot, scope, computedHiddenFiles), [files, hideRoot, scope, computedHiddenFiles]);
    const [collapsedFolders, setCollapsedFolders] = useState(() => new Set());
    // reset collapsed folders when the list of files changes
    useEffect(() => {
        setCollapsedFolders(new Set());
    }, [files]);
    const filteredFileList = useMemo(() => {
        const list = [];
        let lastDepth = Number.MAX_SAFE_INTEGER;
        for (const fileOrFolder of fileList) {
            const depth = fileOrFolder.depth;
            // if the depth is equal we reached the end of the collaped group
            if (lastDepth === depth) {
                lastDepth = Number.MAX_SAFE_INTEGER;
            }
            // ignore collapsed folders
            if (collapsedFolders.has(fileOrFolder.id)) {
                lastDepth = Math.min(lastDepth, depth);
            }
            // ignore files and folders below the last collapsed folder
            if (lastDepth < depth) {
                continue;
            }
            list.push(fileOrFolder);
        }
        return list;
    }, [fileList, collapsedFolders]);
    function toggleCollapseState(id) {
        setCollapsedFolders((prevSet) => {
            const newSet = new Set(prevSet);
            if (newSet.has(id)) {
                newSet.delete(id);
            }
            else {
                newSet.add(id);
            }
            return newSet;
        });
    }
    return (_jsx("div", { className: classNames(className, 'transition-theme bg-tk-elements-fileTree-backgroundColor'), children: filteredFileList.map((fileOrFolder) => {
            switch (fileOrFolder.kind) {
                case 'file': {
                    return (_jsx(File, { selected: selectedFile === fileOrFolder.fullPath, file: fileOrFolder, onClick: () => onFileSelect?.(fileOrFolder.fullPath) }, fileOrFolder.id));
                }
                case 'folder': {
                    return (_jsx(Folder, { folder: fileOrFolder, collapsed: collapsedFolders.has(fileOrFolder.id), onClick: () => toggleCollapseState(fileOrFolder.id) }, fileOrFolder.id));
                }
            }
        }) }));
}
export default FileTree;
function Folder({ folder: { depth, name }, collapsed, onClick }) {
    return (_jsx(NodeButton, { className: "group transition-theme bg-tk-elements-fileTree-folder-backgroundColor hover:bg-tk-elements-fileTree-folder-backgroundColorHover text-tk-elements-fileTree-folder-textColor hover:text-tk-elements-fileTree-folder-textColor", depth: depth, iconClasses: classNames('text-tk-elements-fileTree-folder-iconColor group-hover:text-tk-elements-fileTree-folder-iconColorHover', {
            'i-ph-folder-simple-duotone': collapsed,
            'i-ph-folder-open-duotone': !collapsed,
        }), onClick: onClick, children: name }));
}
function File({ file: { depth, name }, onClick, selected }) {
    return (_jsx(NodeButton, { className: classNames('group transition-theme', {
            'bg-tk-elements-fileTree-file-backgroundColor hover:bg-tk-elements-fileTree-file-backgroundColorHover text-tk-elements-fileTree-file-textColor hover:text-tk-elements-fileTree-file-textColorHover': !selected,
            'bg-tk-elements-fileTree-file-backgroundColorSelected text-tk-elements-fileTree-file-textColorSelected': selected,
        }), depth: depth, iconClasses: classNames('i-ph-file-duotone', {
            'text-tk-elements-fileTree-file-iconColor group-hover:text-tk-elements-fileTree-file-iconColorHover': !selected,
            'text-tk-elements-fileTree-file-iconColorSelected': selected,
        }), onClick: onClick, "aria-pressed": selected, children: name }));
}
function NodeButton({ depth, iconClasses, onClick, className, 'aria-pressed': ariaPressed, children }) {
    return (_jsxs("button", { className: `flex items-center gap-2 w-full pr-2 border-2 border-transparent text-faded ${className ?? ''}`, style: { paddingLeft: `${12 + depth * NODE_PADDING_LEFT}px` }, onClick: () => onClick?.(), "aria-pressed": ariaPressed === true ? 'true' : undefined, children: [_jsx("div", { className: classNames('scale-120 shrink-0', iconClasses) }), _jsx("span", { children: children })] }));
}
function buildFileList(files, hideRoot, scope, hiddenFiles) {
    const folderPaths = new Set();
    const fileList = [];
    const defaultDepth = hideRoot ? 0 : 1;
    if (!hideRoot) {
        fileList.push({ kind: 'folder', name: '/', depth: 0, id: 0 });
    }
    for (const filePath of files) {
        if (scope && !filePath.startsWith(scope)) {
            continue;
        }
        const segments = filePath.split('/').filter((s) => s);
        const fileName = segments.at(-1);
        if (!fileName || isHiddenFile(filePath, fileName, hiddenFiles)) {
            continue;
        }
        let currentPath = '';
        for (let depth = 0; depth < segments.length; ++depth) {
            const name = segments[depth];
            const fullPath = (currentPath += `/${name}`);
            if (depth === segments.length - 1) {
                fileList.push({
                    kind: 'file',
                    id: fileList.length,
                    name,
                    fullPath,
                    depth: depth + defaultDepth,
                });
            }
            else if (!folderPaths.has(fullPath)) {
                folderPaths.add(fullPath);
                fileList.push({
                    kind: 'folder',
                    id: fileList.length,
                    name,
                    depth: depth + defaultDepth,
                });
            }
        }
    }
    return fileList;
}
function isHiddenFile(filePath, fileName, hiddenFiles) {
    return hiddenFiles.some((pathOrRegex) => {
        if (typeof pathOrRegex === 'string') {
            return fileName === pathOrRegex;
        }
        return pathOrRegex.test(filePath);
    });
}
